package org.usgs.itis.itis_service;

import java.util.List;
import java.util.Iterator;
// the service classes used come from the data package
import org.usgs.itis.itis_service.data.xsd.*;

/**
 * test method to access the ITIS Web Service using code generated from the WSDL
 * by the Sun Java wsimport tool
 *
 * The general methodology is:
 *
 *  1. create an instance of the service
 *  2. get the port endpoint from the service
 *  3. get the data from the port endpoint
 *  4. use the data (you must use getValue() to get to the actual data values
 *     from the returned object)
 *
 * You must examine the generated code and/or the web service documentation to
 * find the methods to use in steps 3 and 4.
 *
 * An IDE with method completion suggestions is a big help here.
 *
 * @author ahampson@usgs.gov
 *
 */
public class WS_Test
{
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) 
    {
        // default data
        String tsn = "180543";
        String srchKey = "bear";
        Boolean byTsn = true;

        System.out.println();
        System.out.println();

        //
        // first check for input from the command line
        // if found, determine whether it's a TSN or search key
        //
        if(args != null && args.length > 0)
        {
            // try to convert the input argument to a TSN.
            // if it works, we'll do TSN related stuff.
            // if not, we'll search for common name
            try
            {
                int tsnVal = Integer.parseInt(args[0]);
                tsn = args[0];
            }
            catch (NumberFormatException ex)
            {

                byTsn = false;
                srchKey = args[0];
            }
        }

        //
        // 1. create an instance of the service
        //
        ITISService itisService = new ITISService();

        //
        // 2. get the port endpoint from the service
        //
        ITISServicePortType svcPort = itisService.getITISServiceHttpSoap12Endpoint();

        //
        // 3. get the data from the port endpoint
        //
        GetDescriptionResponse description = svcPort.getDescription();

        //
        // 4. use the data
        //
        String desc = description.getReturn().getValue().getDescription().getValue();
        System.out.println("Service Description: ");
        System.out.println("    " + desc);
        System.out.println();

        if(byTsn)
        {
            //
            // 3. get the data from the port endpoint
            //
            SvcFullRecord record = svcPort.getFullRecordFromTSN(tsn);
            SvcTaxonRankInfo rankInfo = record.getTaxRank().getValue();
            SvcScientificName sciName = record.getScientificName().getValue();
            SvcCommonNameList commonNameList = record.getCommonNameList().getValue();
            List<SvcCommonName> commonNames = commonNameList.getCommonNames();
            SvcParentTsn parentTSN = record.getParentTSN().getValue();

            //
            // 4. use the data (you must use getValue() to get to the actual data values)
            //
            System.out.println("Full Record Data for TSN " + tsn);
            System.out.println("    Scientific name: " + sciName.getCombinedName().getValue());
            System.out.println("    Common names: ");
            if(commonNames != null && !commonNames.isEmpty())
            {
                Iterator<SvcCommonName> it = commonNames.iterator();
                while(it.hasNext())
                {
                    SvcCommonName name = it.next();
                    if(name != null)
                    {
                        System.out.print("        TSN: " + name.getTsn().getValue());
                        System.out.print(" has Vernacular: " + name.getCommonName().getValue());
                        System.out.println(" in Language: " + name.getLanguage().getValue());
                    }
                    else
                    {
                        System.out.println("None");
                        break;
                    }
                }
                System.out.println();
            }
            else
            {
                System.out.println("None");
            }

            System.out.print("    Taxonomic Rank: " + rankInfo.getRankName().getValue());
            System.out.println(" which is Rank ID: " + rankInfo.getRankId().getValue());
            System.out.print("    Kingdom: " + rankInfo.getKingdomName().getValue());
            System.out.println(" which is Kingdom ID: " + rankInfo.getKingdomId().getValue());
            System.out.println("    The Parent TSN is: " + parentTSN.getParentTsn().getValue());
            System.out.println();
        }
        else
        {
            //
            // 3. Search for the common names matching the search key
            //
            SvcCommonNameList commonNameList = svcPort.searchByCommonName(srchKey);

            //
            // 4. Get the CommonName list and print it out
            //
            List<SvcCommonName> commonNames = commonNameList.getCommonNames();
            System.out.println("Common names found for " + srchKey + ": ");
            if(commonNames != null && !commonNames.isEmpty())
            {
                Iterator<SvcCommonName> it = commonNames.iterator();
                while(it.hasNext())
                {
                    SvcCommonName name = it.next();
                    if(name != null)
                    {
                        System.out.print("        TSN: " + name.getTsn().getValue());
                        System.out.print(" has Vernacular: " + name.getCommonName().getValue());
                        System.out.println(" in Language: " + name.getLanguage().getValue());
                    }
                    else
                    {
                        System.out.println("None");
                        break;
                    }
                }
            }
            else
            {
                System.out.println("None");
            }
        }
        System.out.println();
        System.out.println();
    }
}
